/* 
 * File:   ILtc2600.cpp
 * Author: claude
 * 
 * Created on 20 décembre 2016, 10:24
 */

#include <iostream>
#include <sstream>
#include <iomanip>

#include "ILtc2600.h"

ILtc2600::ILtc2600() {
}

ILtc2600::ILtc2600(const ILtc2600& orig) {
}

ILtc2600::~ILtc2600() {
}

/**
 * Méthode associée à la commande "Power Down n" du Ltc2600
 * 
 * @param n
 * @return 
 */
int ILtc2600::powerDownDacReg(dacID_t n) {
    uint8_t data[ 3 ] = {0, 0, 0};
    int ret = -1;
    
    data[ 0 ] = (static_cast<uint8_t> (dacCmd_t::POWER_DOWN_N) << 4)
            | (static_cast<uint8_t> (n)) 
            ;
    

    cout << "cmd : POWER_DOWN_N" << endl
         << "ch  : " << toString(n) << endl
         << "val : DON'T CARE" << endl;
    
    if( sendFrame(data) == 0 ) {
        ret = 0;
    }
    
    return ret;
}

/**
 * Méthode associée à la commande "Update (Power Up) DAC Register n" du Ltc2600
 * 
 * @param n
 * @return 
 */
int ILtc2600::powerUpDacReg(dacID_t n) {
    uint8_t data[ 3 ] = {0, 0, 0};
    int ret = -1;

    data[ 0 ] = (static_cast<uint8_t> (dacCmd_t::UPDATE_DAC_REGISTER_N) << 4)
            | (static_cast<uint8_t> (n))
            ;

    cout << "cmd : UPDATE_DAC_REGISTER_N" << endl
         << " ch : " << toString(n) << endl
         << "val : DON'T CARE" << endl
         ;
         
    if( sendFrame(data) == 0 ) {
        ret = 0;
    }
    
    return ret;
}

/**
 * Méthode associée aux commandes "Write to and Update (Power Up) n" et 
 * "Write to Input Register n, Update (Power Up) All n" du Ltc2600
 * @param n Identifiant du/des DAC(s) [DAC_A...DAC_H, DAC_ALL]
 * @param value Valeur 16 bits à écrire dans le "Input Register" et le
 *              "DAC Register" du/des DAC(s)
 * @param shouldUpdateAll Booléen qui permet de cibler l'une ou l'autre commande
 *                        false -> "Write to and Update (Power Up) n"
 *                        true -> Write to Input Register n, Update (Power Up) All n"
 * @return 
 */
int ILtc2600::updateDacReg(dacID_t n, unsigned short value, bool shouldUpdateAll) {
    uint8_t data[ 3 ] = {0, 0, 0};
    dacCmd_t cmd;
    int ret = -1;

    cmd = shouldUpdateAll
            ? dacCmd_t::WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL_N
            : dacCmd_t::WRITE_TO_AND_UPDATE_N
            ;

    data[ 0 ] = (static_cast<uint8_t> (cmd) << 4) | (static_cast<u_int8_t> (n));
    data[ 1 ] = static_cast<uint8_t>(value >> 8);
    data[ 2 ] = static_cast<uint8_t>(value & 0x00ff);

    if (shouldUpdateAll) {
        cout << "cmd : WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL_N" << endl
              << " ch : " << toString(n) << endl
              << "val : " << toString(value) << endl
              ;
    } else {
        cout << "cmd : WRITE_TO_AND_UPDATE_N" << endl
             << " ch : " << toString(n) << endl
             << "val : " << toString(value) << endl
             ;
    }
    
    if( sendFrame(data) == 0 ) {
        ret = 0;
    }
    
    return ret;

}

/**
 * Méthode associée à la commande "Write to Input Register n" du Ltc2600.
 * 
 * @param n Identifiant du/des DAC(s) [DAC_A...DAC_H, DAC_ALL]
 * @param value Valeur 16 bits à écrire dans le "Input Register" du/des DAC(s)
 * @return 
 */
int ILtc2600::writeInputReg(dacID_t n, unsigned short value) {
    uint8_t data[ 3 ] = {0, 0, 0};
    int ret = -1;
    
    data[ 0 ] = (static_cast<uint8_t> (dacCmd_t::WRITE_TO_INPUT_REGISTER_N) << 4)
            | (static_cast<uint8_t> (n))
            ;
    data[ 1 ] = static_cast<uint8_t>(value >> 8);   
    data[ 2 ] = static_cast<uint8_t>(value & 0x00ff);
 

    cout << "cmd : WRITE_TO_INPUT_REGISTER_N" << endl
         << " ch : " << toString(n) << endl
         << "val : " << toString(value) << endl
         ;  
    
    if( sendFrame(data) == 0 ) {
        ret = 0;
    }
    
    return ret;
}

string ILtc2600::toString(dacID_t n) {
    int ch = static_cast<int> (n);
    ostringstream oss;
    
    if ( ch != 15 ) {
        oss << "DAC " << static_cast<char>('A'+ch) << " (pin " << (ch/4)*10 + (ch%4) + 2 << ")";
    } else {
        oss << "All DACs (pins : 2, 3, 4, 5, 12, 13, 14, 15)";
    }

    return oss.str();
}

string ILtc2600::toString(unsigned short value) {
    ostringstream oss;
        
    oss << value << " (" << hex << showbase << value << ") -> " << left << dec << setw(8) << setprecision(6) << value*5.0/65535 << "V";
    
    return oss.str();
}

