/* 
 * File:   ILtc2600.h
 * Author: claude
 *
 * Created on 20 décembre 2016, 10:24
 */

#ifndef ILTC2600_H
#define ILTC2600_H

#include <cstdint>
#include <stdlib.h>
#include <stdint.h>

#include <string>

using namespace std;

/**
 * Enumération pour identifier les DACs du Ltc2600
 */
enum dacID_t {
    DAC_A, DAC_B, DAC_C, DAC_D, DAC_E, DAC_F, DAC_ALL=15
};

/**
 * Enumération pour les commandes disponibles du Ltc2600
 */
enum class dacCmd_t {
    WRITE_TO_INPUT_REGISTER_N, UPDATE_DAC_REGISTER_N
    , WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL_N, WRITE_TO_AND_UPDATE_N
    , POWER_DOWN_N, NO_OPERATION
};

/**
 * Classe abstraite (-> Interface) qui définit les services que doivent
 * proposer les classes d'interface "concrètes" avec le composant réel ou le
 * simulateur (-> class StubLtc2600)
 */
class ILtc2600 {
public:
    ILtc2600();
    ILtc2600(const ILtc2600& orig);
    virtual ~ILtc2600();
    int writeInputReg(dacID_t n, unsigned short value); // Write to Input Register n
    int powerUpDacReg(dacID_t n); // Update (Power Up) DAC Register n
    int powerDownDacReg(dacID_t n); // Power Down n
    int updateDacReg(dacID_t n, unsigned short value, bool shouldUpdateAll); // Write to and Update (Power Up) n & Write to Input Register n, Update (Power Up) All n
private:
    virtual int sendFrame(uint8_t * theFrame) = 0;
    string toString(dacID_t n);
    string toString(unsigned short value);
};

#endif /* ILTC2600_H */

