/* 
 * File:   StubLtc2600.cpp
 * Author: claude
 * 
 * Created on 20 décembre 2016, 11:25
 */

#include <iostream>
#include <iomanip>

#include "VirLtc2600.h"

using namespace std;

/**
 * Constructeur de la classe d'émulation du Ltc2600.
 * 
 * La méthode s'assure que l'IHM du simulateur de Ltc2600 est en cours d'exécution. 
 */
VirLtc2600::VirLtc2600() {
    FILE *fp;
    char line[ 4096 ];

    // Tenter de récupérer le PID de l'IHM du simulateur de Ltc2600
    fp = popen("pidof virltc2600", "r");
    if (fp == NULL) {
        printf("Echec d'exécution de la commande : 'pidof'\n");
        exit(1);
    }

    // Lire et mémoriser le PID
    fgets(line, sizeof (line) - 1, fp);
    pid = atoi(line);
    if (pid == 0) {
        printf("!! Le simulateur de DAC n'est pas en cours d'exécution !!\n");
        exit(1);
    }

    pclose(fp);

}

/**
 * Constructeur de copie.
 * 
 * @param orig
 */
VirLtc2600::VirLtc2600(const VirLtc2600& orig) {
}

/**
 * Destructeur.
 */
VirLtc2600::~VirLtc2600() {
}



/**
 * Méthode qui envoie la trame de 32 bits au simulateur par l'intermédiaire d'un
 *  signal temps réel (-> seul signal qui permet d'envoyer une donnée).
 * @param msg
 * @return 
 */
int VirLtc2600::sendFrame(u_int8_t * theData) {

    _spiFrame.sival_int = theData[ 0 ] << 16 | theData[ 1 ] << 8 | theData[ 2 ];
    
    cout << "Envoi de la trame [0x" << setw(2) << setfill('0') << hex
         << static_cast<int>(theData[ 0 ]) << "][0x" << setw(2)
         << static_cast<int>(theData[ 1 ]) << "][0x" << setw(2)
         << static_cast<int>(theData[ 2 ]) << "]"
         << " au simulateur de DAC (pid : " << dec << pid << ")"
         << endl;
    
    sigqueue(pid, SIGRTMIN, _spiFrame);

    return 0;
}