/* 
 * File:   SendLtc2600Emul.cpp
 * Author: claude
 *
 * Created on 20 décembre 2016, 12:12
 */

#include <cstdlib>
#include <stdio.h>
#include <getopt.h>
#include <errno.h>
#include <limits.h>
#include <string.h>

#include <sys/time.h>

#include <VirLtc2600.h>
#include <RealLtc2600.h>

using namespace std;

/* Drapeau positionné par l'option ‘--simulator’ */
static int simulator_flag = 0;

/**
 * 
 * @param num
 * @param id
 * @return 
 */
int getDacId(int num, dacID_t *id) {

    int ret = -1;

    if (((num >= 0) && (num <= 7) || (num == 15))) {
        *id = static_cast<dacID_t> (num);
        ret = 0;
    }

    return ret;
}

/**
 * 
 * @param progName
 */
void displayUsage(const char * progName) {
    printf("\nusage : %s  [-s|--simulator] -c|--cmd COMMAND -n|--numdac DAC [-d|--data DATA]\n", progName);
    printf("avec :\n");
    printf(". COMMAND : n° de la commande à exécuter sur le DAC\n"
        "   0 :￼Write to Input Register n\n"
        "   1 : Update (Power Up) DAC Register n\n"
        "   2 : Write to Input Register n\n"
        "   3 : Update (Power Up) All n Write to and Update (Power Up) n\n"
        "   4 : PowerDown n\n"
        "  15 : No operation\n"
        );
    printf(". DAC : n° du DAC cible\n"
        "   0...7 :￼DACs A à H\n"
        "      15 : Tous les DACs\n"
        );
    printf(". DATA : valeur 16bits à envoyer sur le DAC (0...65335 ou 0x0...0xFFFF)\n");
    
    exit(1);
}

/**
 * 
 * @param argc
 * @param argv
 * @return 
 */
int main(int argc, char** argv) {
    int opt;
    dacID_t dacChannel;
    unsigned short dacValue;
    int i;
    ILtc2600 * myDAC;
    unsigned int cmd;
    unsigned int dacnum;
    unsigned int value;
    dacID_t dacid;
    char *endptr;

    while (1) {
        static struct option long_options[] = {
            {"simulator", no_argument, 0, 's'},
            {"cmd", required_argument, 0, 'c'},
            {"numdac", required_argument, 0, 'n'},
            {"data", required_argument, 0, 'd'},
            {"help", no_argument, 0, 'h'},            
            {0, 0, 0, 0}
        };

        /* getopt_long stores the option index here. */
        int option_index = 0;

        opt = getopt_long(argc, argv, "sc:n:d:",
                long_options, &option_index);

        /* Detect the end of the options. */
        if (opt == -1)
            break;

        switch (opt) {
            case 0:
                if (long_options[option_index].flag != 0)
                    break;
                //printf("option %s", long_options[option_index].name);
                //if (optarg)
                //    printf(" with arg %s", optarg);
                //printf("\n");
                break;

            case 's':
                simulator_flag = 1;
                break;

            case 'c':
                //printf("option -c with value `%s'\n", optarg);
                cmd = atoi(optarg);
                break;

            case 'n':
                //printf("option -n with value `%s'\n", optarg);
                if (getDacId(atoi(optarg), &dacid) != 0) {
                    puts("Numero DAC invalide !");
                    exit(1);
                }
                break;

            case 'd':
                //printf("option -d with value `%s'\n", optarg);
                if (strncmp(optarg, "0x", 2) == 0) {
                    optarg = &optarg[2];
                    value = strtol(optarg, &endptr, 16);
                } else {
                    value = strtol(optarg, &endptr, 10);
                }
                if ((errno == ERANGE && (value == LONG_MAX || value == LONG_MIN))
                        || (errno != 0 && value == 0)) {
                    perror("strtol");
                    exit(EXIT_FAILURE);
                }

                if (endptr == optarg) {
                    fprintf(stderr, "No digits were found\n");
                    exit(EXIT_FAILURE);
                }
                break;

            case 'h':
            case '?':
                displayUsage(argv[0]);
                break;

            default:
                abort();
        }
    }

    if (simulator_flag) {
        myDAC = new VirLtc2600();
    } else {
        myDAC = new RealLtc2600();
    }

    switch (cmd) {
        case 0:
            myDAC->writeInputReg(dacid, value);
            break;

        case 1:
            myDAC->powerUpDacReg(dacid);
            break;

        case 2:
            myDAC->updateDacReg(dacid, value, true);
            break;

        case 3:
            myDAC->updateDacReg(dacid, value, false);
            break;

        case 4:
            myDAC->powerDownDacReg(dacid);
            break;

        default:
            puts("Commande inconnue");
            exit(1);
            break;
    }

    return 0;
}
